﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Management;
using System.Text;

namespace ProcessList
{
    class Program
    {
        static void Main(string[] args)
        {
            try
            {
                switch(args[0])
                {
                    case "adv": AdvFilter(args[1]); break;
                    case "api": ApiFilter(args[1]); break;
                    case "demo": Demo(); break;
                    case "wmi": WmiFilter(args[1]); break;
                }
            }
            catch (Exception ex)
            {
                Console.Error.WriteLine($"error: {ex.Message}");
            }
        }

        static void Demo()
        {
            var sw = Stopwatch.StartNew();

            foreach (var proc in Process.GetProcesses())
            {
                Console.WriteLine($"{proc.ProcessName} -> {ProcessListHelper.ProcessHelper.ExtractCommandLine(proc)}");
            }

            Console.WriteLine($"\n{sw.ElapsedMilliseconds}ms elapsed.");
        }

        static void WmiFilter(string procname)
        {
            var sw = new System.Diagnostics.Stopwatch();
            var WMI = new ManagementClass("Win32_Process");
            //while (true)
            {
                sw.Start();
                var Instances = WMI.GetInstances();
                int n1 = 0;
                int n2 = 0;
                foreach (var item in Instances)
                {
                    n1++;
                    string name = item["Name"].ToString().ToLower();
                    if (String.Compare(name, procname) != 0) continue;
                    n2++;
                }
                sw.Stop();
                Console.WriteLine(String.Format("{0} Tasks {1} Perls => {2} msec", n1, n2, sw.ElapsedMilliseconds.ToString()));
                sw.Reset();
            }
        }

        static void AdvFilter(string procname)
        {
            var sw = Stopwatch.StartNew();

            var mgmtObjSearcher = new ManagementObjectSearcher($"SELECT Name,CommandLine FROM Win32_Process WHERE Name='{procname}'");

            int count = mgmtObjSearcher.Get().Count;

            Console.WriteLine($"{count} Perls => {sw.ElapsedMilliseconds} msec");
        }

        static void ApiFilter(string procname)
        {
            var sw = Stopwatch.StartNew();
            foreach(var proc in Process.GetProcesses()
                .Where(x => string.Equals(x.ProcessName, procname, StringComparison.InvariantCultureIgnoreCase)))
            {
                Console.WriteLine($"{proc.ProcessName} -> {ProcessListHelper.ProcessHelper.ExtractCommandLine(proc)}");
            }
            Console.WriteLine($"\n{sw.ElapsedMilliseconds}ms elapsed.");
        }
    }
}
