// This is the main DLL file.

#include "stdafx.h"

#include <Windows.h>
#include <winternl.h>
#include <vector>

#include "ProcessListHelper.h"

namespace ProcessListHelper {

  typedef NTSTATUS(NTAPI* _NtQueryInformationProcess)(
    HANDLE ProcessHandle,
    DWORD ProcessInformationClass,
    PVOID ProcessInformation,
    DWORD ProcessInformationLength,
    PDWORD ReturnLength);

  class ScopedProcessHandle
  {
    HANDLE _processHandle;
  public:
    explicit ScopedProcessHandle(System::Diagnostics::Process^ process)
      : _processHandle(OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, FALSE, process->Id))
    {
    }
    ~ScopedProcessHandle()
    {
      if (_processHandle != nullptr)
      {
        CloseHandle(_processHandle);
      }
    }
    bool valid() const
    {
      return _processHandle != nullptr;
    }
    PVOID GetPebAddress()
    {
      _NtQueryInformationProcess NtQueryInformationProcess = (_NtQueryInformationProcess)GetProcAddress(GetModuleHandleA("ntdll.dll"), "NtQueryInformationProcess");
      PROCESS_BASIC_INFORMATION pbi = { 0 };
      NtQueryInformationProcess(_processHandle, 0, &pbi, sizeof(pbi), NULL);
      return pbi.PebBaseAddress;
    }
    bool ReadProcessMemory(const void* base_addr, void* buffer, std::size_t length)
    {
      return ::ReadProcessMemory(_processHandle, base_addr, buffer, length, nullptr);
    }
    ScopedProcessHandle(const ScopedProcessHandle&) = delete;
    ScopedProcessHandle& operator=(const ScopedProcessHandle&) = delete;
  };

  String^ ProcessHelper::ExtractCommandLine(System::Diagnostics::Process^ process)
  {
    ScopedProcessHandle processHandle(process);

    if(!processHandle.valid())
      return String::Empty;

    PVOID pebAddress = processHandle.GetPebAddress();

    PVOID rtlUserProcParamsAddress;
    if (!processHandle.ReadProcessMemory(&(((_PEB*)pebAddress)->ProcessParameters), &rtlUserProcParamsAddress, sizeof(rtlUserProcParamsAddress)))
      return String::Empty;

    UNICODE_STRING commandLine;
    if (!processHandle.ReadProcessMemory(&(((_RTL_USER_PROCESS_PARAMETERS*)rtlUserProcParamsAddress)->CommandLine), &commandLine, sizeof(commandLine)))
      return String::Empty;

    std::vector<WCHAR> szCmdLine(commandLine.MaximumLength, L'\0');

    if (!processHandle.ReadProcessMemory(commandLine.Buffer, szCmdLine.data(), szCmdLine.size()))
      return String::Empty;

    return gcnew String(szCmdLine.data());
  }

}